@@ -4,31 +4,23 @@
 #'
 #' @param bibfile metadata file to process, either \code{"DATA.bib"} or
 #'        \code{"SOFTWARE.bib"}.
+#' @param quiet whether to suppress messages reporting progress.
 #'
 #' @note
-#' This is a helper function for \code{\link{taf.bootstrap}} and can also be
-#' used in a custom \verb{bootstrap.R} script. It should be called within the
-#' \file{bootstrap} directory that contains the metadata file.
+#' This is a helper function for \code{\link{taf.bootstrap}}. It is called
+#' within the \file{bootstrap} directory that contains the metadata file.
 #'
-#' Within each metadata entry, the \dfn{source} field specifies where data or
-#' software originate from. There are four types of values that can be used in
-#' the source field:
-#' \enumerate{
-#' \item GitHub reference of the form \verb{owner/repo[/subdir]@ref},
-#'       identifying a specific version of an R package. A fixed reference such
-#'       as a tag, release, or SHA-1 hash is recommended. Branch names, such as
-#'       \verb{master}, are pointers that are subject to change, and are
-#'       therefore not reliable as long-term references.
-#' \item URL starting with \verb{http} or \verb{https}, identifying a file to
-#'       download.
-#' \item Relative path starting with \file{initial}, identifying the location of
-#'       a file or folder provided by the user.
-#' \item Special value \code{file}, indicating that the metadata key points to a
-#'       file location.
-#' }
+#' A metadata file contains one or more entries that use a general BibTeX
+#' format:
+#'
+#' \preformatted{@Type{key,
+#'   field = {value},
+#'   ...,
+#' }}
 #'
 #' Consider, for example, the following metadata entry from a \verb{DATA.bib}
 #' file:
+#'
 #' \preformatted{@Misc{PLE7DFleet_2016.txt,
 #'   originator = {WGNSSK},
 #'   year       = {2016},
@@ -36,6 +28,7 @@
 #'   period     = {1987-2015},
 #'   source     = {file},
 #' }}
+#'
 #' Here, a data file is described using the \verb{@Misc} entry type and the
 #' string following the entry type is called a \dfn{key}. The next fields state
 #' that this file was prepared by the North Sea working group in 2016 and it
@@ -47,6 +40,27 @@
 #' equivalent to specifying the relative path:
 #' \verb{source = {initial/data/PLE7DFleet_2016.txt}}.
 #'
+#' The \dfn{source} field specifies where data or software originate from. There
+#' are four types of values that can be used in the source field:
+#'
+#' \enumerate{
+#' \item GitHub reference of the form \verb{owner/repo[/subdir]@ref},
+#'       identifying a specific version of an R package. A fixed reference such
+#'       as a tag, release, or SHA-1 hash is recommended. Branch names, such as
+#'       \verb{master}, are pointers that are subject to change, and are
+#'       therefore not reliable as long-term references.
+#' \item URL starting with \verb{http} or \verb{https}, identifying a file to
+#'       download.
+#' \item Relative path starting with \file{initial}, identifying the location of
+#'       a file or directory provided by the user.
+#' \item Special value \code{file}, indicating that the metadata key points to a
+#'       file location.
+#' \item Special value \code{script}, indicating that an R script should be run
+#'       to fetch data files from a web service. The metadata key is used both
+#'       to identify the script \file{bootstrap/\emph{key}.R} and target
+#'       directory \file{bootstrap/data/\emph{key}}.
+#' }
+#'
 #' Another example metadata entry is from a \verb{SOFTWARE.bib} file:
 #' \preformatted{@Manual{FLAssess,
 #'   author  = {Laurence T Kell},
@@ -56,28 +70,34 @@
 #'   version = {2.6.2, released 2018-07-18},
 #'   source  = {flr/FLAssess@v2.6.2},
 #' }}
+#'
 #' This entry describes a specific version of an R package that is required for
 #' the TAF analysis. It is similar, but not identical, to the output from the R
 #' command \verb{citation("FLAssess")}. The version field specifies the version
 #' number and release date, with a corresponding GitHub reference. When an R
 #' package is not an official release but a development version, the version and
 #' source may look like this,
+#'
 #' \preformatted{  version = {2.6.3, committed 2018-10-09},
 #'   source  = {flr/FLAssess@f1e5acb},}
+#'
 #' or this:
+#'
 #' \preformatted{  version = {0.5.4 components branch, committed 2018-03-12},
 #'   source  = {fishfollower/SAM/stockassessment@25b3591},}
+#'
 #' For development versions like these, the version number itself may not be
 #' important or accurate, but the branch name and commit date may be
 #' informative. The 7-character SHA reference code is a pointer to the exact
 #' version of the package required for the analysis.
 #'
 #' As a final metadata example, we look at a software entry that is not an R
-#' package. It is made available as a zip archive, containing the model source
-#' code (sole.tpl) and executables for different platforms (sole, sole.exe). The
-#' model does not have an explicit version number, so the \verb{version} field
-#' contains the year in which the model is used, along with the date when the
-#' source code was last modified:
+#' package. It is made available as a directory \file{sole} containing the model
+#' source code (\verb{sole.tpl}) and executables for different platforms
+#' (\verb{sole}, \verb{sole.exe}). The model does not have an explicit version
+#' number, so the version field contains the year in which the model is used,
+#' along with the date when the source code was last modified:
+#'
 #' \preformatted{@Article{sole,
 #'   author  = {G. Aarts and J.J. Poos},
 #'   year    = {2009},
@@ -88,9 +108,20 @@
 #'   pages   = {763-771},
 #'   doi     = {10.1093/icesjms/fsp033},
 #'   version = {2016, last modified 2016-04-27},
-#'   source  = {initial/software/sole.zip},
+#'   source  = {initial/software/sole},
 #' }}
 #'
+#' The source field can specify multiple URLs to download, separated by
+#' newlines. To shorten the source entries, the \dfn{prefix} field can be useful
+#' to specify a prefix that is common for all source entries.
+#'
+#' The \dfn{dir} field is optional and creates a directory to place files in. If
+#' the dir field is used, it can only have the value \verb{dir = {TRUE}} and the
+#' resulting directory will be named after the metadata key. The dir field is
+#' mainly useful when two or more data files that need to be downloaded have the
+#' same name. It is implied and not necessary to set \verb{dir = TRUE} when
+#' \verb{source = {script}} or when the source field specifies multiple URLs.
+#'
 #' In summary, the metadata are similar to bibliographic entries, with the
 #' important addition of source directives that guide the bootstrap procedure to
 #' set up data files and software.
@@ -102,8 +133,6 @@
 #' create initial draft versions of \file{DATA.bib} and \file{SOFTWARE.bib}
 #' metadata files.
 #'
-#' \code{\link{period}} pastes two years to form a \code{period} string.
-#'
 #' \code{\link{icesTAF-package}} gives an overview of the package.
 #'
 #' @examples
@@ -113,83 +142,42 @@
 #' }
 #'
 #' @importFrom bibtex read.bib
-#' @importFrom remotes install_github parse_repo_spec
 #'
 #' @export
 
-process.bib <- function(bibfile)
+process.bib <- function(bibfile, quiet=FALSE)
 {
   type <- if(bibfile == "DATA.bib") "data"
           else if(bibfile == "SOFTWARE.bib") "software"
           else stop("bibfile must be 'DATA.bib' or 'SOFTWARE.bib'")
 
   entries <- if(file.exists(bibfile)) read.bib(bibfile) else list()
+  dups <- anyDuplicated(names(entries))
+  if(dups)
+    stop("Duplicated key: '", names(entries)[dups], "'")
 
   for(bib in entries)
   {
-    ## Prepare dir
-    dir <- if(is.null(bib$bundle)) type else file.path(type, bib$bundle)
-    mkdir(dir)
+    key <- attr(bib, "key")
 
     ## If source contains multiple files then split into vector
     bib$source <- trimws(unlist(strsplit(bib$source, "\\n")))
-    bib$source <- sub(",$", "", bib$source)  # remove trailing comma
+    bib$source <- sub(",$", "", bib$source)     # remove trailing comma
+    bib$source <- bib$source[bib$source != ""]  # remove empty strings
 
     ## Add prefix
     bib$source <- paste0(bib$prefix, bib$source)
 
-    ## Case 1: R package on GitHub
-    if(grepl("@", bib$source[1]))
-    {
-      spec <- parse_repo_spec(bib$source)
-      url <- paste0("https://api.github.com/repos/",
-                    spec$username, "/", spec$repo, "/tarball/", spec$ref)
-      targz <- paste0(spec$repo, "_", spec$ref, ".tar.gz")
-      if(!file.exists(file.path(dir, targz)))
-        suppressWarnings(download(url, destfile=file.path(dir, targz)))
-      mkdir("library")
-      ## Need to check manually and force=TRUE, since the install_github()
-      ## built-in checks get confused if package is installed in another library
-      if(already.in.taf.library(spec))
-      {
-        pkg <- basename(file.path(spec$repo, spec$subdir))
-        message("Skipping install of '", pkg, "'.")
-        message("  Version '", spec$ref,
-                "' is already in the local TAF library.")
-      }
-      else
-      {
-        install_github(bib$source, lib="library", dependencies=FALSE,
-                       upgrade=FALSE, force=TRUE)
-      }
-    }
-    ## Case 2: File to download
-    else if(grepl("^http", bib$source[1]))
-    {
-      sapply(bib$source, download, dir=dir)
-    }
-    ## Case 3: File to copy
-    else
-    {
-      ## Shorthand notation: source = {file} means key is a filename
-      if(bib$source[1] == "file")
-        bib$source[1] <- file.path("initial", dir, attr(bib,"key"))
-      sapply(bib$source, cp, to=dir)
-    }
+    ## Prepare dir, where bib$dir starts as: TRUE, FALSE, string, or NULL
+    bib$dir <- as.logical(bib$dir)  # is now TRUE, FALSE, NA, or logical(0)
+    if(identical(bib$dir, NA))
+      stop("parsing entry '", key, "' - dir should be TRUE or unspecified")
+    dir <- if(identical(bib$dir, TRUE) ||
+              length(bib$source) > 1 ||
+              bib$source[1] == "script")
+             file.path(type, key) else type
+    mkdir(dir)  # target directory
+
+    process.inner(bib, dir, quiet)
   }
 }
-
-## Check whether requested package is already installed in the TAF library
-
-#' @importFrom utils installed.packages packageDescription
-
-already.in.taf.library <- function(spec)
-{
-  pkg <- basename(file.path(spec$repo, spec$subdir))
-  sha.bib <- spec$ref
-  sha.inst <- if(pkg %in% row.names(installed.packages("library")))
-                packageDescription(pkg, "library")$RemoteSha else NULL
-  sha.inst <- substring(sha.inst, 1, nchar(sha.bib))
-  out <- identical(sha.bib, sha.inst)
-  out
-}
