@@ -3,12 +3,12 @@
 #' Set up data files and software required for the analysis. Model configuration
 #' files are also set up, if found.
 #'
-#' @param clean whether to \code{\link{clean}} all TAF directories
-#'        (\verb{bootstrap}, \verb{data}, \verb{model}, \verb{output},
-#'        \verb{report}) before initiating the bootstrap procedure.
+#' @param clean whether to \code{\link{clean}} directories during the bootstrap
+#'        procedure.
 #' @param config whether to process configuration files.
 #' @param data whether to process data.
 #' @param software whether to process software.
+#' @param quiet whether to suppress messages reporting progress.
 #'
 #' @note
 #' This function should be called from the top directory of a TAF analysis. It
@@ -25,10 +25,6 @@
 #'       processed with \code{\link{process.bib}}.
 #' }
 #'
-#' To override this default bootstrap procedure, the user can create a custom
-#' \verb{bootstrap.R} script. If this script is found, the \code{taf.bootstrap}
-#' function runs that script instead of the default bootstrap procedure.
-#'
 #' After the bootstrap procedure, data and software have been documented and
 #' are ready to be used in the subsequent analysis. Specifically, the procedure
 #' populates up to four new directories:
@@ -55,40 +51,56 @@
 #'
 #' @export
 
-taf.bootstrap <- function(clean=TRUE, config=TRUE, data=TRUE, software=TRUE)
+taf.bootstrap <- function(clean=TRUE, config=TRUE, data=TRUE, software=TRUE,
+                          quiet=FALSE)
 {
   if(clean)
-    clean(c("bootstrap", "data", "model", "output", "report"))
+    clean()
 
-  if(file.exists("bootstrap.R"))
-  {
-    sourceTAF("bootstrap.R")
-  }
-  else
-  {
-    if(!dir.exists("bootstrap"))
-      return(invisible(NULL))  # nothing to do
+  if(!dir.exists("bootstrap"))
+    return(invisible(NULL))  # nothing to do
+  if(!quiet)
     msg("Bootstrap procedure running...")
 
-    ## Work inside bootstrap
-    setwd("bootstrap"); on.exit(setwd(".."))
+  ## Work inside bootstrap
+  setwd("bootstrap"); on.exit(setwd(".."))
 
-    ## 1  Process config
-    if(config && dir.exists("initial/config"))
-      cp("initial/config", ".")
-
-    ## 2  Process data
-    if(data)
-      process.bib("DATA.bib")
-
-    ## 3  Process software
-    if(software)
-      process.bib("SOFTWARE.bib")
-
-    ## Remove empty folders
-    rmdir(c("config", "data", "library", "software"))
-    rmdir("library:", recursive=TRUE)  # this directory name can appear in Linux
-    msg("Bootstrap procedure done")
-    invisible(NULL)
+  ## 1  Process config
+  if(config && dir.exists("initial/config"))
+  {
+    if(!quiet)
+      message("Processing config")
+    if(clean)
+      clean("config")
+    cp("initial/config", ".")
   }
+
+  ## 2  Process data
+  if(data)
+  {
+    if(!quiet)
+      message("Processing DATA.bib")
+    if(clean)
+      clean("data")
+    process.bib("DATA.bib", quiet=quiet)
+  }
+
+  ## 3  Process software
+  if(software)
+  {
+    if(!quiet)
+      message("Processing SOFTWARE.bib")
+    if(clean)
+      clean(c("library", "software"))
+    process.bib("SOFTWARE.bib", quiet=quiet)
+  }
+
+  ## Remove empty folders
+  rmdir(c("config", "data", "library", "software"))
+  rmdir("library:", recursive=TRUE)  # this directory name can appear in Linux
+
+  if(!quiet)
+    msg("Bootstrap procedure done")
+
+  invisible(NULL)
 }
